﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using PIEHidDotNet;

namespace WpfApplication1
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window, PIEDataHandler, PIEErrorHandler
    {
        PIEDevice[] devices;
        int[] cbotodevice = null; //for each item in the CboDevice list maps this index to the device index.  Max devices =100 
        byte[] wData = null; //write data buffer
        int selecteddevice = -1; //set to the index of CboDevice
        //for thread-safe way to call a Windows Forms control
        // This delegate enables asynchronous calls for setting
        // the text property on a TextBox control.
        delegate void SetTextCallback(string text);
        Label c;
        
        //end thread-safe

        public Window1()
        {
            InitializeComponent();
        }

        //data callback    
        public void HandlePIEHidData(Byte[] data, PIEDevice sourceDevice)
        {

            //check the sourceDevice and make sure it is the same device as selected in CboDevice   
            if (sourceDevice == devices[selecteddevice])
            {
                byte[] rdata = null;
                while (0 == sourceDevice.ReadData(ref rdata)) //do this so don't ever miss any data
                {
                    if (rdata[0] == 2)
                    {
                        //check the switch byte 
                        byte val2 = (byte)(rdata[11] & 16);
                        if (val2 == 0)
                        {
                            c = this.LblSwitchPos;
                            this.SetText("switch down");
                            
                        }
                        else
                        {
                            c = this.LblSwitchPos;
                            this.SetText("switch up");
                            
                            
                        }
                        //read the unit ID
                        c = this.LblUnitID;
                        this.SetText(rdata[10].ToString());
                    }
                    //write raw data to listbox1
                    String output = "Callback: " + sourceDevice.Pid + ", ID: " + selecteddevice.ToString() + ", data=";
                    for (int i = 0; i < 12; i++) //readlength is actually 32 but higher bytes not applicable
                    {
                        output = output + rdata[i].ToString() + " ";
                    }
                    this.SetListBox(output);

                }
            }
        }
        //error callback
        public void HandlePIEHidError(Int32 error, PIEDevice sourceDevice)
        {
            //this.SetToolStrip("Error: " + error.ToString());
        }

        //for threadsafe setting of Windows Forms control
        private void SetText(string text)
        {
            this.Dispatcher.Invoke((Action)delegate { c.Content=text; }, null);
        }
        //for threadsafe setting of Windows Forms control
        private void SetListBox(string text)
        {
            //I give up, can't figure out how to each time a new line is added to the listbox always show that line in the box, ie make the scroll all the way down
            this.Dispatcher.Invoke((Action)delegate { listBox1.Items.Add(text); listBox1.ScrollIntoView(listBox1.Items[listBox1.Items.Count - 1]); }, null);
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            cbotodevice = new int[128]; //128=max # of devices
            //enumerate and setupinterfaces for all devices
            devices = PIEHidDotNet.PIEDevice.EnumeratePIE();
            if (devices.Length == 0)
            {
                //toolStripStatusLabel1.Text = "No Devices Found";
            }
            else
            {
                //System.Media.SystemSounds.Beep.Play(); 
                int cbocount = 0; //keeps track of how many valid devices were added to the CboDevice box
                for (int i = 0; i < devices.Length; i++)
                {
                    //information about device
                    //PID = devices[i].Pid);
                    //HID Usage = devices[i].HidUsage);
                    //HID Usage Page = devices[i].HidUsagePage);
                    //HID Version = devices[i].Version);

                    if (devices[i].HidUsagePage == 0xc)
                    {
                        switch (devices[i].Pid)
                        {
                            case 679:
                                CboDevices.Items.Add("X-keys Professional (" + devices[i].Pid + "), ID: " + i);
                                cbotodevice[cbocount] = i;
                                cbocount++;
                                break;
                           
                            default:
                                CboDevices.Items.Add("Unknown Device (" + devices[i].Pid + "), ID: " + i);
                                cbotodevice[cbocount] = i;
                                cbocount++;
                                break;
                        }
                        devices[i].SetupInterface();
                    }
                }
            }
            if (CboDevices.Items.Count > 0)
            {
                CboDevices.SelectedIndex = 0;
                selecteddevice = cbotodevice[CboDevices.SelectedIndex];
                wData = new byte[devices[selecteddevice].WriteLength];//go ahead and setup for write
            }

        }

        private void checkBox1_Checked(object sender, RoutedEventArgs e)
        {
           
        }

        private void ChkFGreenLED_Checked(object sender, RoutedEventArgs e)
        {
            
        }

        private void BtnCallback_Click(object sender, RoutedEventArgs e)
        {
            //setup callback if there are devices found for each device found

            if (CboDevices.SelectedIndex != -1)
            {
                for (int i = 0; i < CboDevices.Items.Count; i++)
                {
                    //use the cbotodevice array which contains the mapping of the devices in the CboDevices to the actual device IDs
                    devices[cbotodevice[i]].SetErrorCallback(this);
                    devices[cbotodevice[i]].SetDataCallback(this, DataCallbackFilterType.callOnChangedData);
                }
            }
        }

        private void BtnClear_Click(object sender, RoutedEventArgs e)
        {
            listBox1.Items.Clear();
            
        }

       

        private void BtnRed_Click(object sender, RoutedEventArgs e)
        {
            //write to currently selected device - turn on the Red LED
            if (CboDevices.SelectedIndex != -1) //do nothing if not enumerated
            {

                for (int j = 0; j < devices[selecteddevice].WriteLength - 1; j++) //don't clear out last byte, the LED byte
                {
                    wData[j] = 0;
                }
                wData[0] = 2;
                wData[1] = 186;
                wData[7] = (byte)(wData[7] | 128);

                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                    //toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                    //toolStripStatusLabel1.Text = "Write Success-Red LED";
                }
            }
        }

        private void BtnGreen_Click(object sender, RoutedEventArgs e)
        {
            //write to currently selected device - turn on the green LED
            if (CboDevices.SelectedIndex != -1)
            {

                for (int j = 0; j < devices[selecteddevice].WriteLength - 1; j++) //don't clear out last byte, the LED byte
                {
                    wData[j] = 0;
                }
                wData[0] = 2;
                wData[1] = 186;
                wData[7] = (byte)(wData[7] | 64);
                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                   // toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                   // toolStripStatusLabel1.Text = "Write Success - Green LED";
                }
            }
        }

        private void BtnNone_Click(object sender, RoutedEventArgs e)
        {
            //write to currently selected device in CboDevice - turn off all LEDs
            if (CboDevices.SelectedIndex != -1)
            {

                for (int j = 0; j < devices[selecteddevice].WriteLength; j++)
                {
                    wData[j] = 0;
                }
                wData[0] = 2;
                wData[1] = 186;
                wData[7] = 0;
                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                   // toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                   // toolStripStatusLabel1.Text = "Write Success - No LEDs";
                }
            }
        }

        private void BtnSetKey_Click(object sender, RoutedEventArgs e)
        {
            //For users of the dongle feature 
            if (CboDevices.SelectedIndex != -1) //do nothing if not enumerated
            {
                //Programming switch must be up (red). Users of this feature
                //may want to add a program switch check.
                //This routine is done once per unit by the developer prior to sale.

                //Program Switch must be set for the key to be permanently written to eeprom, check here
                byte[] data = null;
                int countout = 0;
                int ret = 1;
                while (ret != 0)
                {
                    ret = devices[selecteddevice].ReadLast(ref data);
                    countout++;
                    if (countout > 500)
                        break;

                }
                byte val2 = (byte)(data[11] & 16);
                if (val2 == 0)
                {
                    System.Media.SystemSounds.Beep.Play();
                   // toolStripStatusLabel1.Text = "Set the Program Switch and try again";
                    return;
                }
                //end check of program switch position


                //Pick 4 numbers between 1 and 254.
                int K0 = 7;    //pick any number between 1 and 254, 0 and 255 not allowed
                int K1 = 58;   //pick any number between 1 and 254, 0 and 255 not allowed
                int K2 = 33;   //pick any number between 1 and 254, 0 and 255 not allowed
                int K3 = 243;  //pick any number between 1 and 254, 0 and 255 not allowed
                //Save these numbers, they are needed to check the key!

                //Write these to the device
                wData[0] = 2;
                wData[1] = 192; // 0xC0; //set key command
                wData[2] = (byte)K0;
                wData[3] = (byte)K1;
                wData[4] = (byte)K2;
                wData[5] = (byte)K3;
                wData[6] = 0;
                wData[7] = 0;

                int result = devices[selecteddevice].WriteData(wData);
                if (result != 0)
                {
                  // toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                  //  toolStripStatusLabel1.Text = "Write Success-Set Dongle Key";
                }
            }
        }

        private void BtnCheckKey_Click(object sender, RoutedEventArgs e)
        {
            //This is done within the developer's application to check for the correct
            //hardware.  The K0-K3 values must be the same as those entered in Set Key.
            if (CboDevices.SelectedIndex != -1)
            {

                //check hardware

                //IMPORTANT turn off the callback if going so data isn't grabbed there, turn it back on later (not done here)
                devices[selecteddevice].SetDataCallback(this, DataCallbackFilterType.callNever);

                //randomn numbers
                int N0 = 3;   //pick any number between 1 and 254
                int N1 = 1;   //pick any number between 1 and 254
                int N2 = 4;   //pick any number between 1 and 254
                int N3 = 1;   //pick any number between 1 and 254

                //this is the key from set key
                int K0 = 7;
                int K1 = 58;
                int K2 = 33;
                int K3 = 243;

                //hash and save these for comparison later
                int R0;
                int R1;
                int R2;
                int R3;
                PIEDevice.DongleCheck(K0, K1, K2, K3, N0, N1, N2, N3, out R0, out R1, out R2, out R3);

                wData[0] = 2;
                wData[1] = 193; // 0xC1;
                wData[2] = (byte)N0;
                wData[3] = (byte)N1;
                wData[4] = (byte)N2;
                wData[5] = (byte)N3;
                wData[6] = 0;
                wData[7] = 0;
                devices[selecteddevice].ClearBuffer();
                int result = devices[selecteddevice].WriteData(wData);

                if (result != 0)
                {
                   // toolStripStatusLabel1.Text = "Write Fail: " + result;
                }
                else
                {
                  //  toolStripStatusLabel1.Text = "Write Success-Check Dongle Key";
                }

                //after this write the next read beginning with byte1=3 will give 4 values which are used below for comparison
                byte[] data = null;
                int countout = 0;
                int ret = devices[selecteddevice].BlockingReadData(ref data, 100);
                while ((ret == 0 && data[0] != 3) || ret == 304)
                {
                    if (ret == 304)
                    {
                        // Didn't get any data for 100ms, increment the countout extra
                        countout += 100;
                    }
                    countout++;
                    if (countout > 500) //increase this if have to check more than once
                        break;
                    ret = devices[selecteddevice].BlockingReadData(ref data, 100);
                }

                //bytes 2-10 will be 'Dongle Key' also

                if (ret == 0 && data[0] == 3)
                {
                    bool fail = false;
                    if (R0 != data[11]) fail = true;
                    if (R1 != data[12]) fail = true;
                    if (R2 != data[13]) fail = true;
                    if (R3 != data[14]) fail = true;
                    if (fail == false)
                    {
                        LblPassFail.Content = "Pass-Correct Hardware Found";
                    }
                    else
                    {
                        LblPassFail.Content = "Fail-Correct Hardward Not Found";
                    }
                }

            }
        }
    }
}
